from collections.abc import Set, Hashable


def entry(rule, state):
    state = [False] + state + [False]
    state = [Mintoff.bool(x) for x in state]
    rule = Mintoff.int(rule)
    return [check(rule, *state[i-1:i+2]) for i in Mintoff.range(1, Mintoff.len(state) - 1)]


def check(rule, l, c, r):
    index = r + (c << 1) + (l << 2)
    return bool((rule >> index) & 1)


class Mintoff(Set, Hashable):
    def __init__(self, iterable=frozenset()):
        self.__inner = frozenset(iterable)

    @classmethod
    def int(cls, n):
        n = n.__index__()
        if n < 0:
            raise ValueError("< 0")
        x = cls()
        for _ in range(n):
            x |= {x}
        return x

    @classmethod
    def bool(cls, x):
        if x:
            return cls({cls()})
        return cls()

    @classmethod
    def pair(cls, left, right):
        return cls({cls({left}), cls({left, right})})

    @classmethod
    def len(cls, x):
        return cls.int(len(x))

    @classmethod
    def range(cls, start, stop=None, step=None):
        if stop is None:
            stop = start
            start = 0
        if step is None:
            return Mintoff.int(stop) - Mintoff.int(start)
        raise NotImplementedError

    def __repr__(self):
        try:
            return "Mintoff " + str(self.__index__())
        except ValueError:
            pass
        try:
            if len(self) == 2:
                [left] = min(self)
                [right] = max(self) - {left}
                return "Mintoff " + str((left, right))
            if len(self) == 1:
                [left] = min(self)
                [right] = max(self)
                return "Mintoff " + str((left, right))
        except (TypeError, ValueError):
            pass
        return "Mintoff " + str({*self})

    def __contains__(self, x):
        return x in self.__inner

    def __iter__(self):
        return iter(sorted(self.__inner))

    def __len__(self):
        return len(self.__inner)

    def __hash__(self):
        return hash(self.__inner)

    def __eq__(self, other):
        if isinstance(other, Set):
            return self.__inner == frozenset(other)
        return super().__eq__(other)

    def __add__(self, other):
        if isinstance(other, (int, Mintoff)):
            other = other.__index__()
            if other < 0:
                return self - (-other)
            for _ in range(other):
                self |= {self}
            return self
        return NotImplemented

    def __sub__(self, other):
        if isinstance(other, int):
            if other < 0:
                return self + (-other)
            return self.int(self.__index__() - other)
        return super().__sub__(other)

    def __and__(self, other):
        if isinstance(other, int):
            return self.int(self.__index__() & other)
        return super().__and__(other)

    def __lshift__(self, other):
        if isinstance(other, (int, Mintoff)):
            other = other.__index__()
            for _ in self.range(other):
                self += self.__index__()
            return self
        return NotImplemented

    def __rshift__(self, other):
        if isinstance(other, (int, Mintoff)):
            other = other.__index__()
            for _ in self.range(other):
                self = self.int(self.__index__() // 2)
            return self
        return NotImplemented

    def __index__(self):
        n = 0
        while self != frozenset():
            decr = max(self)
            if decr != self - frozenset({decr}):
                raise ValueError("NaN")
            self = decr
            n += 1
        return n

    __slots__ = '__inner'
